setwd("./WGCNA")
library(WGCNA)
library(flashClust)
library(iterators)
library(ggplot2)
result<-read.table("Tc_expression.csv", sep = ",",header=T,row.names = 1,stringsAsFactors=F) 

enableWGCNAThreads()
femData <- result
datExpr = as.data.frame(t(femData))
powers = c(c(1:10), seq(from = 12, to=20, by=2))
sft=pickSoftThreshold(datExpr,dataIsExpr = TRUE,
                      powerVector = powers,
                      corFnc = cor,
                      corOptions = list(use = 'p'),
                      networkType = 'unsigned')
pdf("power_plot.pdf")
par(mfrow = c(1,2))

plot(x=sft$fitIndices[,1],
     y=-sign(sft$fitIndices[,3])*sft$fitIndices[,2],
     xlab="Soft Threshold (power)",
     ylab="Scale Free Topology Model Fit, signed R^2",type="n",
     main = paste("Scale independence"))
text(sft$fitIndices[,1], -sign(sft$fitIndices[,3])*sft$fitIndices[,2],
     labels=powers,cex=0.9,col="red")
abline(h=0.80,col="red")

plot(sft$fitIndices[,1], sft$fitIndices[,5],
     xlab="Soft Threshold (power)", ylab="Mean Connectivity", type="n",
     main = paste("Mean connectivity"))
text(sft$fitIndices[,1], sft$fitIndices[,5], labels=powers, cex=0.9,col="red")
dev.off()

softPower=6 
TOM = TOMsimilarityFromExpr(datExpr,
                            networkType = 'signed',
                            TOMType = 'signed',
                            power = softPower)
colnames(TOM) =rownames(TOM) = rownames(femData)

dissTOM=1-TOM
geneTree = hclust(as.dist(dissTOM),method='average') 
minModuleSize = 50 #gene numbers
dynamicMods = cutreeDynamic(dendro = geneTree,
                            distM = dissTOM,
                            method='hybrid',
                            deepSplit = 2,
                            pamRespectsDendro = FALSE,
                            minClusterSize = minModuleSize) 
table(dynamicMods) 
dynamicColors = labels2colors (dynamicMods) 
table(dynamicColors)
#Module Eigengenes
MEList = moduleEigengenes (datExpr, colors = dynamicColors) 
MEs = MEList$eigengenes
MEDiss = 1-cor(MEs); 
METree = hclust(as.dist(MEDiss), method = "average"); 
pdf("correlation.pdf")
plotEigengeneNetworks(MEs, "Eigengene adjacency heatmap",
                      marHeatmap = c(3,4,2,2),
                      plotDendrograms = FALSE,
                      xLabelsAngle = 90)
plot(METree, main = "Clustering of module eigengenes", xlab = "", sub = "")
abline(h=0.5, col="red")
dev.off()

merge_modules = mergeCloseModules(datExpr, dynamicColors, cutHeight = 0.2, verbose = 3)
mergedColors = merge_modules$colors;
mergedMEs = merge_modules$newMEs; 
pdf("module cluster.pdf")
plotDendroAndColors(geneTree, cbind(dynamicColors, mergedColors),
                    c("Dynamic Tree Cut", "Merged dynamic"),
                    dendroLabels = FALSE, hang = 0.03,
                    addGuide = TRUE, guideHang = 0.05)
dev.off()
cl=c()
module_gene=c()
module_colors=unique(mergedColors)
for (color in module_colors){
  module=rownames(femData)[which(mergedColors==color)]
  group1=rep(color,times=length(module))
  cl=c(cl,group1)
  module_gene=c(module_gene,module)
}
my_module=data.frame("colour"=cl,"gene"=module_gene)
write.csv(my_module,"Modules_gene.csv")

diss1=1-TOM
hier1=flashClust(as.dist(diss1), method='average')
geneTree = hclust(as.dist(diss1), method = "average");
#set the diagonal of the dissimilarity to NA
diag(diss1) = NA
TOMplot(diss1, hier1, as.character(mergedColors))

#Eigengene significance
group=factor(rep(c("Ma","Mi","Tu","Fl","Ag","Py","Ven"),each=3),levels = c("Ma","Mi","Tu","Fl","Ag","Py","Ven")) 
datTraits = data.frame(samples=rownames(datExpr),subtype=group)
design=model.matrix(~0 + datTraits$subtype)
colnames(design)=levels(factor(datTraits$subtype))
MEs0 = moduleEigengenes(datExpr, mergedColors)$eigengenes
MEs = orderMEs(MEs0); 
moduleTraitCor = cor(MEs, design , use = "p") 
nSamples = nrow(datExpr)
moduleTraitPvalue = corPvalueStudent(moduleTraitCor, nSamples)

textMatrix = paste(signif(moduleTraitCor, 2), "\n(",signif(moduleTraitPvalue, 1), ")", sep = "");
dim(textMatrix) = dim(moduleTraitCor)
pdf('group_Modules_heatmap.pdf',width=8,height=12)
labeledHeatmap(Matrix = moduleTraitCor,
               xLabels = colnames(design),
               xLabelsAngle = 0,
               yLabels = row.names(moduleTraitCor),
               ySymbols = row.names(moduleTraitCor),
               colorLabels = TRUE,
               colors = blueWhiteRed(50),
               textMatrix = textMatrix,
               setStdMargins = FALSE,
               cex.text = 1,
               zlim = c(-1,1),
               main = paste("Module-trait relationships"))
dev.off()



#edge and node list
for (color in module_colors){
  gene=my_module$gene[which(my_module$colour==color)]
  color_gene=as.matrix( TOM[gene,gene])
  cyt = exportNetworkToCytoscape(color_gene,
                                 edgeFile = paste('CytoscapeInput-edges-', color,'.txt', sep=''),
                                 nodeFile = paste('CytoscapeInput-node-', color,'.txt', sep=''),
                                 weighted = TRUE, threshold = 0, nodeNames = gene, altNodeNames = gene)
}
